/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "DepelShell.hpp"

#include "Ellib.hpp"

#include <memory>

namespace lispel
{
using namespace ellisp;
using namespace ellib;
namespace depel
{

/* ************************************************************************** */
/* ******************************** DepelShell ****************************** */
/* ************************************************************************** */

DepelShell::DepelShell(std::istream& in, std::ostream& out, bool prompt)
  : mainIn(in), parse(), rt(), interact(prompt)
{
  registerLibrary(rt);
  rt.setOutputStream(out);

  parse.setInput(mainIn);
}

bool DepelShell::next(bool ia)
{
  if(ia)
    std::cout << "depel> ";

  try
  {
    std::auto_ptr<Cell> cmd(parse.parseDeclaration());
    rt.process(*cmd);
  } catch(const Parser::EndOfInput& eof)
  {
    return false;
  } catch(const std::runtime_error& exc)
  {
    std::cerr << "Error: " << exc.what() << std::endl;
    // Abort for non-interactive use, otherwise continue the shell
    return ia;
  }

  return true;
}

void DepelShell::include(std::istream& in)
{
  // RAII to temporarily change the parser's input
  class ChangeInput
  {
    private:
      Parser& parse;
      std::istream& old;
    public:
      inline ChangeInput(Parser& p, std::istream& o, std::istream& n)
        : parse(p), old(o)
      {
        parse.setInput(n);
      }
      inline ~ChangeInput()
      {
        parse.setInput(old);
      }
  } changer(parse, mainIn, in);

  while(next(false));
}

} // namespace depel
} // namespace lispel
