/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DEPEL_DEPELSHELL_HPP
#define DEPEL_DEPELSHELL_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Parser.hpp"

#include "Runtime.hpp"

#include <iostream>

namespace lispel
{
namespace depel
{

/* ************************************************************************** */
/* ******************************** DepelShell ****************************** */
/* ************************************************************************** */

/**
 * The command-processor for Depel, reading/parsing input and running it through
 * the Lispel-Runtime.  Optionally it can act fully as Shell, prompting 
 * interactively.
 */
class DepelShell
{

  /* ************************************************************************ */
  public:

    /**
     * Construct and set-up the shell
     * @param in The input stream to parse commands from
     * @param out The output stream to write output-values to
     * @param prompt Use it interactively?
     */
    DepelShell(std::istream& in, std::ostream& out, bool prompt =false);

    // No destructor needed
    
    /**
     * Parse and run one single operation.
     * @return False if end of input reached or runtime-error occured
     */
    inline bool next()
    {
      return next(interact);
    }

    /**
     * Run until end-of-input/error encountered.
     */
    inline void run()
    {
      while(next());
    }

    /**
     * Process an "include" command, temporarily read non-interactive from a
     * given, new input stream.
     * @param in The new input stream
     */
    void include(std::istream& in);

  /* ************************************************************************ */
  private:

    /** The main input stream to read from */
    std::istream& mainIn;

    /** The Parser */
    Parser parse;

    /** The Runtime */
    ellisp::Runtime rt;

    /** Run interactively? */
    const bool interact;

    /**
     * Parse and run from an arbitrary parser source
     * @param ia Interactivity-mode to use
     * @see next()
     */
    bool next(bool ia);

};

} // namespace depel
} // namespace lispel

#endif // Header-guard
