/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLIB_CELLTYPES_HPP
#define ELLIB_CELLTYPES_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Type.hpp"
#include "Elob.hpp"

#include <stdexcept>
#include <sstream>

namespace lispel
{
namespace ellisp
{
  class Runtime;
  class Atom;
}
using namespace ellisp;
namespace ellib
{

/**
 * Support of the built-in types used in Cells: number, integer, string.  number
 * and string form their own trees, integer is a sub-type of number as all
 * integers are numbers.
 */
namespace CellTypes
{

  /**
   * Register those types with a Runtime
   */
  void define(Runtime& rt);

  /**
   * The Type for Cell::numberType / Cell::stringType
   * @param T The type of backing object
   */
  template<typename T>
    class TopType : public Type
  {
    public:
      inline TopType(const Atom& name, const Runtime& r)
        : Type(name, r)
      {}
      virtual void destroyData(void* val) const
      {
        delete static_cast<T*>(val);
      }
      virtual Elob castValueTo(const Elob& val, Ref t) const
      {
        // Is already at the top of the hierarchy
        std::ostringstream msg;
        msg << getName() << " is already at the top of the type hierarchy ";
        msg << "and can not be casted to " << t->getName();
        throw std::runtime_error(msg.str());
      }
      virtual std::ostream& print(std::ostream& out, const void* val) const
      {
        out << *static_cast<const T*>(val);
        return out;
      }
  };

  /**
   * The Type for Cell::intType.
   */
  class IntType : public Type
  {
    public:
      explicit IntType(const Runtime& r);
      virtual void destroyData(void* val) const;
      virtual Elob castValueTo(const Elob& val, Ref t) const;
      virtual std::ostream& print(std::ostream& out, const void* val) const;
  };

} // namespace CellType

// The code also defines the Elob::fromXXX and Elob::getXXX methods
// Also, Runtime::getXXXType

} // namespace ellib
} // namespace lispel

#endif // Header-guard
