/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLIB_CONDITIONS_HPP
#define ELLIB_CONDITIONS_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Function.hpp"
#include "Type.hpp"

#include <iostream>

namespace lispel
{
namespace ellisp
{
  class Runtime;
  class Elob;
}
using namespace ellisp;
namespace ellib
{

/**
 * The boolean type, if-function and logic operators.
 */
namespace Conditions
{

  /**
   * Register those functions with a Runtime
   */
  void define(Runtime& rt);

  /**
   * Type for boolean value.
   */
  class Boolean : public Type
  {
    public:
      Boolean(const Runtime& r);
      // Destroy data and castValueTo is ok in default implementation
      virtual std::ostream& print(std::ostream& out, const void* val) const;
  };

  #if 0
  If-function replaced by built-in expression for lazy evaluation
  /**
   * The if-function: (if condition then-value else-value)
   */
  class If : public Function
  {
    public:
      explicit inline If(const Runtime& r)
        : Function(r)
      {}
      virtual bool accepts(const Arguments& vals) const;
      virtual Elob eval(const Arguments& vals) const;
  };
  #endif

  // Callback-methods for logic operations
  Elob logicNot(const Runtime& rt, const Arguments& args);
  Elob logicAnd(const Runtime& rt, const Arguments& args);
  Elob logicOr(const Runtime& rt, const Arguments& args);

} // namespace Conditions

// Implementation has code for Runtime/Elob's boolean-methods

} // namespace ellib
} // namespace lispel

#endif // Header-guard
