/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLIB_SIDEEFFECTS_HPP
#define ELLIB_SIDEEFFECTS_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Function.hpp"
#include "Type.hpp"
#include "SideEffect.hpp"
#include "Elob.hpp"
#include "Runtime.hpp"

#include <vector>
#include <cassert>

namespace lispel
{
using namespace ellisp;
namespace ellib
{

/**
 * Library-support for handling side-effect "values"
 */
namespace SideEffects
{

  /**
   * Register those functions with a Runtime
   */
  void define(Runtime& rt);

  /**
   * Type for side-effect values
   */
  class SideEffectVal : public Type
  {
    public:
      SideEffectVal(const Runtime& r);
      virtual void destroyData(void* val) const;
      // No cast possible, we are at top of hierarchy
      virtual std::ostream& print(std::ostream& out, const void* val) const;
  };

  /**
   * Side-effect class used to represent a sequence.
   * This is a non-lazy version of _sequence.
   */
  class Sequence : public SideEffect
  {

    public:

      explicit inline Sequence(const Runtime& r)
        : SideEffect(r), acts()
      {}

      // No destructor needed

      virtual void perform() const;

      inline void add(const Elob& e)
      {
        assert(e.getType()==rt.getSideEffectType());
        acts.push_back(e);
      }

    private:

      typedef std::vector<Elob> actArr;
      actArr acts; // The actions to perform, must be exact of side-effect type

  };

  /**
   * Sequence-creation function
   */
  class CreateSeq : public Function
  {
    public:
      explicit inline CreateSeq(const Runtime& r)
        : Function(r)
      {}
      virtual bool accepts(const Arguments& vals) const;
      virtual Elob eval(const Arguments& vals) const;
  };

  // Callback for the do-"meta"-action
  void doAction(const Runtime& rt, Arguments& vals);

} // namespace SideEffects

// Code for Elob::xxxSideEffect is in implementation

} // namespace ellib
} // namespace lispel

#endif // Header-guard
