/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLIB_STRINGS_HPP
#define ELLIB_STRINGS_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Function.hpp"

namespace lispel
{
namespace ellisp
{
  class Runtime;
  class Elob;
}
using namespace ellisp;
namespace ellib
{

/**
 * Functions for string operations.
 */
namespace Strings
{

  /**
   * Register those functions with a Runtime
   */
  void define(Runtime& rt);

  /**
   * Concatenation takes variable number of arguments, so use our own function
   * object.
   */
  class Concatenation : public Function
  {
    public:
      explicit inline Concatenation(const Runtime& r)
        : Function(r)
      {}
      virtual bool accepts(const Arguments& vals) const;
      virtual Elob eval(const Arguments& vals) const;
  };

  // Callback-methods
  Elob length(const Runtime& rt, const Arguments& args);
  Elob substring(const Runtime& rt, const Arguments& args);
  Elob charAt(const Runtime& rt, const Arguments& args);

} // namespace Strings

} // namespace ellib
} // namespace lispel

#endif // Header-guard
