/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLIB_VARIABLES_HPP
#define ELLIB_VARIABLES_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Function.hpp"
#include "Action.hpp"
#include "Type.hpp"

namespace lispel
{
namespace ellisp
{
  class Runtime;
  class Elob;
}
using namespace ellisp;
namespace ellib
{

/**
 * The variable-object and action/method for assignment/type extraction.
 */
namespace Variables
{

  /**
   * Register those functions with a Runtime
   */
  void define(Runtime& rt);

  /**
   * Query for the variable type
   * @param rt The runtime
   * @return The variable type of the given runtime
   */
  Type::Ref getVariableType(const Runtime& rt);

  /**
   * Set the value of a variable
   * @param var The Elob holding the variable (must be one!)
   * @param val The value to set it to
   */
  void setVariable(Elob& var, const Elob& val);

  /**
   * Type for variable; that is, a value simply pointing to some other Elob,
   * thus having a generic Elob which can be modified by an assignment-action.
   */
  class Variable : public Type
  {
    public:
      Variable(const Runtime& r);
      virtual void destroyData(void* val) const;
      // No casting possible, thus default implementation is ok
      virtual std::ostream& print(std::ostream& out, const void* val) const;
  };

  /**
   * Create a new variable holding some value
   */
  class CreateVar : public Function
  {
    public:
      explicit inline CreateVar(const Runtime& r)
        : Function(r)
      {}
      virtual bool accepts(const Arguments& vals) const;
      virtual Elob eval(const Arguments& vals) const;
  };

  /**
   * Function to extract the value from a variable, via callback
   */
  Elob extractValue(const Runtime& rt, const Arguments& vals);

  /**
   * Action to assign a variable a new value
   */
  class Assignment : public Action
  {
    public:
      explicit inline Assignment(const Runtime& r)
        : Action(r)
      {}
      virtual bool accepts(const Arguments& vals) const;
      virtual void perform(Arguments& vals) const;
  };

} // namespace Variables

} // namespace ellib
} // namespace lispel

#endif // Header-guard
