/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_ANYTYPE_HPP
#define ELLISP_ANYTYPE_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Type.hpp"

namespace lispel
{
namespace ellisp
{

class Runtime;
class Elob;

/* ************************************************************************** */
/* ****************************** AnyType *********************************** */
/* ************************************************************************** */

/**
 * The type object used for the special "any" type; no value should ever be
 * allowed to actually be of this type!
 */
class AnyType : public Type
{

  /* ************************************************************************ */
  public:

    /**
     * Create a new AnyType-instance for the given runtime.
     * @param r The runtime
     */
    explicit AnyType(const Runtime& r);

    // No destructor needed
    
    /**
     * Define in the given Runtime, should be non-const version of the one
     * this type was created for!
     * @param r The Runtime to define in, takes ownership over "this"
     */
    void define(Runtime& r);

    // Override type's methods with assert(false)
#ifndef NDEBUG
    virtual void destroyData(void* val) const;
    virtual Elob castValueTo(const Elob& val, Ref t) const;
    virtual std::ostream& print(std::ostream& out, const void* val) const;
#endif /* NDEBUG */

    /**
     * Query the any-type instance of the given runtime.
     * @param rt The runtime
     * @return The anytype of this runtime
     */
    static Type::Ref getAnyType(const Runtime& rt);

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
