/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Atom.hpp"

#include <cassert>

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ******************************* Atom ************************************* */
/* ************************************************************************** */

Atom::Atom(const std::string& str)
{
  const idMapT::iterator pos(map.find(str));
  if(pos==map.end())
  {
    id=nextId++;
    map[str]=id;
  } else
  {
    id=pos->second;
    assert(pos->first==str);
  }
}

const std::string& Atom::str() const
{
  // This does not need performance as much as the other way round
  for(idMapT::const_iterator i(map.begin()); i!=map.end(); ++i)
    if(i->second==id)
      return i->first;
  
  // We should always find our ID!
  assert(false);
}

/* ***************************** Bookkeeping of map ************************* */

int Atom::nextId(0);

Atom::idMapT Atom::map;

} // namespace ellisp
} // namespace lispel
