/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Cast.hpp"
#include "List.hpp"
#include "Cell.hpp"
#include "Runtime.hpp"
#include "Atom.hpp"

#include <stdexcept>

namespace lispel
{
namespace ellisp
{

#define NAME_CAST Atom("_cast")

/* ************************************************************************** */
/* ******************************* Cast ************************************* */
/* ************************************************************************** */

Cast::Cast(const Runtime& r, const List* lst)
  : rt(r)
{
  // We can assume this is a cast, meaning that we've got (_cast ...)
  assert(isCast(lst));
  assert(lst);
  assert(lst->getCar()==NAME_CAST);

  // Type to cast to
  lst=lst->getCdr();
  if(!lst)
    throw std::runtime_error("Cast missing target type!");
  if(lst->getCar().getType()!=Cell::ATOM)
    throw std::runtime_error("Cast has must have type as second cell!");
  targetType=r.queryType(lst->getCar().getAtom());
  if(targetType->isAnyType())
    throw std::runtime_error("Can never cast to any-type!");

  // Value to cast to
  lst=lst->getCdr();
  if(!lst)
    throw std::runtime_error("Cast missing value to cast!");
  value=&lst->getCar();

  // No more arguments
  if(lst->getCdr())
    throw std::runtime_error("Cast with too many cells!");
}

Elob Cast::evaluate() const
{
  // Evaluate the value first
  Elob evalValue(rt.evaluate(*value));

  // Now, do the cast
  return evalValue.castTo(targetType);
}

bool Cast::isCast(const List* lst)
{
  // No empty list!
  if(!lst)
    return false;
  
  // Check name
  if(lst->getCar().getType()!=Cell::ATOM)
    return false;
  if(lst->getCar().getAtom()!=NAME_CAST)
    return false;

  // Yes, it should be cast; malformation is caught in constructor
  return true;
}

} // namespace ellisp
} // namespace lispel
