/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_CAST_HPP
#define ELLISP_CAST_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Type.hpp"

namespace lispel
{
namespace ellisp
{

class List;
class Cell;
class Runtime;
class Elob;

/* ************************************************************************** */
/* ******************************* Cast ************************************* */
/* ************************************************************************** */

/**
 * Simple helper class for explicit casting, used in Runtime.
 */
class Cast
{

  /* ************************************************************************ */
  public:

    /**
     * Construct the cast-object
     * @param r The runtime
     * @param lst The list to "parse"
     */
    Cast(const Runtime& r, const List* lst);

    /**
     * Perform the cast, return the casted object.
     * @return The value with new type
     */
    Elob evaluate() const;

    /**
     * Check if a list is a cast
     * @param lst The list to check
     * @return True if this list is a cast
     */
    static bool isCast(const List* lst);

  /* ************************************************************************ */
  private:

    /** Our runtime */
    const Runtime& rt;

    /** Type to cast to */
    Type::Ref targetType;

    /** Reference to value to cast */
    const Cell* value;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
