/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "ConstDef.hpp"
#include "List.hpp"
#include "Cell.hpp"
#include "Runtime.hpp"

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ******************************* ConstDef ********************************* */
/* ************************************************************************** */

#define NAME_CONSTDEF Atom("_constant")

ConstDef::ConstDef(Runtime& r, const List* lst)
  : rt(r), name(), value(NULL)
{
  // Must be const-def, thus first element must be keyword
  assert(isConstDef(lst));
  assert(lst);

  // Next one is name
  lst=lst->getCdr();
  if(!lst)
    throw std::runtime_error("Expected name of constant");
  if(lst->getCar().getType()!=Cell::ATOM)
    throw std::runtime_error("Name of constant must be Atom");
  name=lst->getCar().getAtom();

  // Then the value
  lst=lst->getCdr();
  if(!lst)
    throw std::runtime_error("Expected value of constant!");
  value=&lst->getCar();

  // Nothing more!
  if(lst->getCdr())
    throw std::runtime_error("Too many elements in constant definition!");
}

void ConstDef::perform()
{
  rt.addConstant(name, rt.evaluate(*value));
}

bool ConstDef::isConstDef(const List* lst)
{
  // We need at least one element
  if(!lst)
    return false;

  // This must be the keyword
  if(lst->getCar().getType()!=Cell::ATOM)
    return false;
  if(lst->getCar().getAtom()!=NAME_CONSTDEF)
    return false;

  return true;
}

} // namespace ellisp
} // namespace lispel
