/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_CONSTDEF_HPP
#define ELLISP_CONSTDEF_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Atom.hpp"

namespace lispel
{
namespace ellisp
{

class List;
class Cell;
class Runtime;

/* ************************************************************************** */
/* ******************************* ConstDef ********************************* */
/* ************************************************************************** */

/**
 * Simple helper class for constant-definition construct.
 */
class ConstDef
{

  /* ************************************************************************ */
  public:

    /**
     * Construct the constant-definition object
     * @param r The runtime
     * @param lst The list to "parse"
     */
    ConstDef(Runtime& r, const List* lst);

    /**
     * Evaluate and perform the definition
     */
    void perform();

    /**
     * Check if a list is a constant definition
     * @param lst The list to check
     * @return True if this list is a constant definition
     */
    static bool isConstDef(const List* lst);

  /* ************************************************************************ */
  private:

    /** Our runtime */
    Runtime& rt;

    /** Name of the constant */
    Atom name;

    /** Reference to value to assign it */
    const Cell* value;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
