/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "ConstTable.hpp"
#include "Runtime.hpp"

#include <sstream>
#include <stdexcept>

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ***************************** ConstTable ********************************* */
/* ************************************************************************** */

void ConstTable::add(const Atom& name, const Elob& val)
{
  const valMap::const_iterator pos(table.find(name));
  if(pos!=table.end())
  {
    std::ostringstream msg;
    msg << "Constant " << name << " defined twice in same scope!";
    throw std::runtime_error(msg.str());
  }

  table[name]=val;
}

const Elob& ConstTable::query(const Atom& name) const
{
  const valMap::const_iterator pos(table.find(name));
  if(pos==table.end())
    throw Runtime::Unknown("Constant "+name.str());

  return pos->second;
}

} // namespace ellisp
} // namespace lispel
