/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_CONSTTABLE_HPP
#define ELLISP_CONSTTABLE_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Atom.hpp"
#include "Elob.hpp"

#include <map>
#include <vector>
#include <cassert>

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ***************************** ConstTable ********************************* */
/* ************************************************************************** */

/**
 * A map of names to constants
 */
class ConstTable
{

  /* ************************************************************************ */
  public:

    /**
     * Create an empty constant table.
     */
    inline ConstTable()
      : table()
    {}

    /**
     * Define a new constant
     * @param name The name
     * @param val The value to associate
     * @throws std::runtime_error if this name is already in the current scope
     */
    void add(const Atom& name, const Elob& val);

    /**
     * Query a value by its name.
     * @param name The name
     * @return The associated value
     * @throws Runtime::Unknown if not defined
     */
    const Elob& query(const Atom& name) const;

  /* ************************************************************************ */
  private:

    /** The mapping name -> value */
    typedef std::map<Atom, Elob> valMap;

    /** The map */
    valMap table;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
