/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Elob.hpp"

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ******************************* Elob ************************************* */
/* ************************************************************************** */

Elob::Elob(Type::Ref t, void* val, const Runtime& rt, bool destr)
  : fields(new Content)
{
  assert(!t->isAnyType());
  fields->type=t;
  fields->data=val;
  fields->rt=&rt;
  fields->destroy=destr;
  fields->parent=NULL;
  fields->refcnt=1;
}

Elob::Elob(Type::Ref t, void* val, const Elob& p)
  : fields(new Content)
{
  assert(!t->isAnyType());
  fields->type=t;
  fields->data=val;
  fields->rt=&p.getRuntime();
  fields->destroy=true;
  fields->parent=new Elob(p);
  fields->refcnt=1;
}

void Elob::release()
{
  // Simply ignore, but destructor of empty Elob might call this
  if(!fields)
    return;

  // Descrement count
  --fields->refcnt;
  assert(fields->refcnt>=0);

  // Free if this was last one
  if(!fields->refcnt)
  {
    if(fields->destroy)
    {
      if(!fields->parent)
        fields->type->destroyData(fields->data);
      else
        delete fields->parent; /* This releases the parent once */
    }

    delete fields;
    fields=NULL; // Now we could re-use this Elob reference object
  }
}

Elob Elob::castTo(Type::Ref t) const
{
  assert(t->isAssignableFrom(getType()));

  // No cast needed
  if(getType()==t)
    return *this;

  // Otherwise, cast
  return getType()->castValueTo(*this, t);
}

std::ostream& operator<<(std::ostream& out, const Elob& e)
{
  return e.getType()->print(out, e.getValue());
}

/* The forXXX and getXXX methods for built-in types are part of the runtime
 * library and thus defined there
 */

} // namespace ellisp
} // namespace lispel
