/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "If.hpp"
#include "List.hpp"
#include "Cell.hpp"
#include "Runtime.hpp"
#include "Elob.hpp"
#include "Atom.hpp"

#include <sstream>
#include <stdexcept>

namespace lispel
{
namespace ellisp
{

#define NAME_IF Atom("_if")

/* ************************************************************************** */
/* ******************************** If ************************************** */
/* ************************************************************************** */

If::If(const Runtime& r, const List* lst)
  : rt(r)
#ifndef NDEBUG
  , condition(NULL), ifValue(NULL), elseValue(NULL)
#endif
{
  assert(isIf(lst));
  assert(lst);

  #define EXTRACT(name, var) \
    lst=lst->getCdr(); \
    if(!lst) \
      throw std::runtime_error("If missing " name); \
    var=&lst->getCar();

  EXTRACT("condition", condition)
  EXTRACT("if-value", ifValue)
  EXTRACT("else-value", elseValue)

  // But nothing more
  if(lst->getCdr())
    throw std::runtime_error("Too many cells in if-expression");
}

Elob If::evaluate() const
{
  const Elob cond(rt.evaluate(*condition));
  if(!rt.getBooleanType()->isAssignableFrom(cond.getType()))
  {
    std::ostringstream msg;
    msg << "If-condition " << cond << " is not of boolean type!";
    throw std::runtime_error(msg.str());
  }

  return rt.evaluate(cond.getBoolean() ? *ifValue : *elseValue);
}

bool If::isIf(const List* lst)
{
  // must be (_if ...)
  if(!lst)
    return false;
  if(lst->getCar().getType()!=Cell::ATOM)
    return false;
  if(lst->getCar().getAtom()!=NAME_IF)
    return false;

  return true;
}

} // namespace ellisp
} // namespace lispel
