/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_IF_HPP
#define ELLISP_IF_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Type.hpp"

namespace lispel
{
namespace ellisp
{

class List;
class Cell;
class Runtime;
class Elob;

/* ************************************************************************** */
/* ******************************** If ************************************** */
/* ************************************************************************** */

/**
 * If-expression doing lazy evaluation of arguments so that recursion is
 * possible with this.
 */
class If
{

  /* ************************************************************************ */
  public:

    /**
     * Construct the if-object
     * @param r The runtime
     * @param lst The list to "parse"
     */
    If(const Runtime& r, const List* lst);

    /**
     * Evaluate the expression
     * @return The value of this expression
     */
    Elob evaluate() const;

    /**
     * Check if a list is an if-expression
     * @param lst The list to check
     * @return True if this list is an if-expression
     */
    static bool isIf(const List* lst);

  /* ************************************************************************ */
  private:

    /** Our runtime */
    const Runtime& rt;

    /** Reference to condition-cell */
    const Cell* condition;

    /** Reference to if-cell */
    const Cell* ifValue;

    /** Reference to else-cell */
    const Cell* elseValue;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
