/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_TYPETABLE_HPP
#define ELLISP_TYPETABLE_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Atom.hpp"
#include "Type.hpp"

#include <map>
#include <cassert>

namespace lispel
{
namespace ellisp
{

/* ************************************************************************** */
/* ***************************** TypeTable ********************************** */
/* ************************************************************************** */

/**
 * The map of types to their names.
 */
class TypeTable
{

  /* ************************************************************************ */
  public:

    /**
     * Create an empty type table
     */
    inline TypeTable()
      : table()
    {}

    /**
     * Destruct the table
     */
    ~TypeTable();

    /**
     * Add a new type
     * @param name The name of it
     * @param t The type object; taken ownership of
     */
    inline void add(const Atom& name, Type* t)
    {
      assert(table.find(name)==table.end());
      table[name]=t;
    }

    /**
     * Query a type by its name.
     * @param name The name
     * @return The type
     * @throws Runtime::Unknown if no such type is found
     */
    Type::Ref query(const Atom& name) const;

  /* ************************************************************************ */
  private:

    /** The mapping name -> types */
    typedef std::map<Atom, Type*> typeMap;

    /** The type map used herein */
    typeMap table;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
