/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_USERACTION_HPP
#define ELLISP_USERACTION_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Action.hpp"
#include "Arguments.hpp"
#include "Atom.hpp"

#include <vector>

namespace lispel
{
namespace ellisp
{

class Runtime;
class List;

/* ************************************************************************** */
/* ****************************** UserAction ******************************** */
/* ************************************************************************** */

/**
 * An action defined "dynamically" in Lispel-code; it consists of a set of
 * arguments (types and names) together with a set of lists forming the body.
 */
class UserAction : public SignatedAction
{

  /* ************************************************************************ */
  public:

    /**
     * Construct a UserAction from a List defining it.  The code will be
     * spliced out the list, modifying it!
     * @param r The Runtime
     * @param lst The List defining the action
     */
    UserAction(const Runtime& r, List* lst);

    /**
     * Destruct, freeing the code-list
     */
    ~UserAction();

    /**
     * Define the action in the given Runtime, should be non-const version
     * of the real runtime.
     * @param r The runtime to define in
     */
    void define(Runtime& r);

    /**
     * Check if a List is an action definition
     * @param lst The list to check
     * @return True if lst defines an action, i.e., is (_action ...)
     */
    static bool isAction(const List* lst);

  /* ************************************************************************ */
  protected:

    virtual void performCasted(Arguments& vals) const;

  /* ************************************************************************ */
  private:

    /** This action's name */
    Atom name;

    /** The argument names of the action */
    ArgNames argNames;

    /** The code to evaluate; list containing the individual commands */
    List* code;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
