/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef ELLISP_USERFUNCTION_HPP
#define ELLISP_USERFUNCTION_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Function.hpp"
#include "Arguments.hpp"
#include "Atom.hpp"

#include <vector>

namespace lispel
{
namespace ellisp
{

class Runtime;
class List;
class Elob;

/* ************************************************************************** */
/* ****************************** UserFunction ****************************** */
/* ************************************************************************** */

/**
 * A function defined "dynamically" in Lispel-code; it consists of a set of
 * arguments (types and names) together with a List evaluated to calculate its
 * result.
 */
class UserFunction : public SignatedFunction
{

  /* ************************************************************************ */
  public:

    /**
     * Construct a UserFunction from a List defining it.  The code will be
     * spliced out the list, modifying it!
     * @param r The Runtime
     * @param lst The List defining the function
     */
    UserFunction(const Runtime& r, List* lst);

    /**
     * Destruct, freeing the code-list
     */
    ~UserFunction();

    /**
     * Define the function in the given Runtime, should be non-const version
     * of the real runtime.
     * @param r The runtime to define in
     */
    void define(Runtime& r);

    /**
     * Check if a List is a function definition
     * @param lst The list to check
     * @return True if lst defines a function, i.e., is (_function ...)
     */
    static bool isFunction(const List* lst);

  /* ************************************************************************ */
  protected:

    virtual Elob evalCasted(const Arguments& vals) const;

  /* ************************************************************************ */
  private:

    /** This function's name */
    Atom name;

    /** The argument names of the function */
    ArgNames argNames;

    /** The code to evaluate; one-element list spliced out */
    List* code;

};

} // namespace ellisp
} // namespace lispel

#endif // Header-guard
