/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TEST_CLASSICEVAL_HPP
#define TEST_CLASSICEVAL_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Runtime.hpp"
#include "Type.hpp"

#include <string>

namespace lispel
{
namespace ellisp
{
  class Elob;
}
using namespace ellisp;
namespace tests
{

/* ************************************************************************** */
/* **************************** ClassicEval ********************************* */
/* ************************************************************************** */

/**
 * A lispel runtime which can be used easily for testing as evaluator of
 * expressions given in classic notation
 */
class ClassicEval : public Runtime
{

  /* ************************************************************************ */
  public:

    /**
     * Construct it; this also defines the CellTypes here.
     */
    ClassicEval();

    /**
     * Process a string
     * @param str The string to process
     */
    void procClassic(const std::string& str);

    /**
     * Evaluate an expression given as string to an Elob and type-check it
     * for an expected type.
     * @param str The string to evaluate as expression
     * @param t The expected type
     * @return The evaluated value
     */
    Elob evalClassic(const std::string& str, Type::Ref t) const;

};

} // namespace tests
} // namespace lispel

#endif // Header-guard
