/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TEST_DECOMPARATOR_HPP
#define TEST_DECOMPARATOR_HPP

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Parser.hpp"

#include <string>

namespace lispel
{
using namespace depel;
namespace tests
{

/* ************************************************************************** */
/* **************************** DeComparator ******************************** */
/* ************************************************************************** */

/**
 * Help-class encapsulating parsing of german language expressions and comparing
 * them to their (hopefully) equivalent Lisp expressions.
 */
class DeComparator
{

  /* ************************************************************************ */
  public:

    /**
     * Construct, possibly being verbose
     * @param v Be verbose?
     */
    explicit inline DeComparator(bool v =false)
      : verbose(v)
    {}

    /**
     * Check if two strings parse the same.  Aborts if not.
     * @param de The de-pel format string
     * @param lisp The lisp format string
     */
    void check(const std::string& de, const std::string& lisp);

  /* ************************************************************************ */
  private:

    /** The parser used */
    Parser p;

    /** Be verbose? */
    const bool verbose;

};

} // namespace tests
} // namespace lispel

#endif // Header-guard
