/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test for checking general performing of actions by the Runtime.  We do not
 * yet need any predefined library-actions, we simply define our own basic one.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Arithmetic.hpp"

#include "Elob.hpp"
#include "Action.hpp"
#include "Atom.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

// The callback for inc: Action to increment a value
void perfInc(const Runtime& rt, Arguments& args)
{
  check(args.size()==1, "Inc called with %d arguments!", args.size());
  check(args.front().getType()==rt.getIntegerType(), "Inc not called with int");

  // Note: This is tightly coupled with Integer-type implementation!
  int* val(static_cast<int*>(args.front().getValue()));
  ++(*val);
}

#define CONSTANT Atom("xxx")

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);

  Signature sig;
  sig.push_back(proc.getIntegerType());
  proc.addAction(Atom("inc"),
    new CallbackAction(proc, sig, &perfInc));

  // First, define the constant we want to increment
  proc.addConstant(CONSTANT, Elob::forInteger(41, proc));

  // Next, evaluate the action
  proc.procClassic("(inc xxx)");

  // The value should have been updated now
  check(proc.queryConstant(CONSTANT).getInteger()==42,
    "Constant not updated properly, is %d instead of %d!",
    proc.queryConstant(CONSTANT).getInteger(), 42);

  // This should not be evaluated as action and should therefore not fail
  proc.procClassic("(Summe 5 xxx)");
  
  return EXIT_SUCCESS;
}
