/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test for the any-type feature; simply defines an any-type-signated method
 * and calls it with completely different arguments.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Conditions.hpp"
#include "Comparisons.hpp"

#include <cstdlib>
#include <iostream>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Conditions::define(proc);
  Comparisons::define(proc);

  // Define function taking any-type
  proc.procClassic("(_function MeineUngleichheit ((Irgendwas a) (Irgendwas b))"
    " (Verneinung (Gleichheit a b)))");

  // Test it
  Elob val;
  val=proc.evalClassic("(MeineUngleichheit 3 5)", proc.getBooleanType());
  check(val.getBoolean(), "Wrong for numbers");
  val=proc.evalClassic("(MeineUngleichheit \"abc\" \"abc\")",
    proc.getBooleanType());
  check(!val.getBoolean(), "Wrong for strings");

  return EXIT_SUCCESS;
}
