/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* arithmetic tests the built-in basic arithmetic: addition, subtraction,
 * multiplication, division and modulo.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Arithmetic.hpp"

#include "Cell.hpp"
#include "Elob.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);

  Elob val;
  #define DO_CHECK(instr, restype, res, format) \
    val=proc.evalClassic(instr, proc.get##restype##Type()); \
    check(val.get##restype()==res, \
      "Result value mismatch, is " format ", should be " format, \
      val.get##restype(), res);
  
  DO_CHECK("(Summe 5 6 2)", Integer, 5+6+2, "%d")
  DO_CHECK("(Summe 4 .2)", Number, 4+.2, "%f")
  DO_CHECK("(Produkt 3)", Integer, 3, "%d")
  DO_CHECK("(Produkt (Summe 1 .1) 5)", Number, (1+.1)*5, "%f")
  DO_CHECK("(Differenz 5 4)", Integer, 5-4, "%d")
  DO_CHECK("(Differenz 5. 3.)", Number, 5.-3., "%f")
  DO_CHECK("(Quotient 5 3)", Integer, 5/3, "%d")
  DO_CHECK("(Quotient 5 2.)", Number, 5/2., "%f")
  DO_CHECK("(Rest 27 4)", Integer, 27%4, "%d")

  return EXIT_SUCCESS;
}
