/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Test.hpp"

#include "Cell.hpp"
#include "List.hpp"

#include <cstdlib>
#include <sstream>
#include <iostream>

using namespace lispel;
using namespace ellisp;
using namespace tests;

using namespace std;

// The string to be parsed as cell originally
const char* INPUT="(abc () -5 -0.42 \"daniel kraft\\\\\\\"patrica\" (a (b) d))";

int main()
{
  // Parse it
  istringstream in(INPUT);
  Cell c;
  in >> c;

  // Do some checks about the structure
  check(c.getType()==Cell::LIST, "Main-cell is no list!");
  const List* l(c.getList());
  #define NEXT_CELL_TYPE(type, name) \
    check(l, "More cells in list expected!"); \
    check(l->getCar().getType()==type, "Next cell should be " name); \
    l=l->getCdr();
  NEXT_CELL_TYPE(Cell::ATOM, "atom")
  NEXT_CELL_TYPE(Cell::LIST, "list")
  NEXT_CELL_TYPE(Cell::INTEGER, "integer")
  NEXT_CELL_TYPE(Cell::NUMBER, "number")
  NEXT_CELL_TYPE(Cell::STRING, "string")

  // Output and check form
  ostringstream out;
  out << c;
  check(out.str()==INPUT, "Re-output mismatches original input!\n%s\n%s",
    out.str().c_str(), INPUT);

  return EXIT_SUCCESS;
}
