/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test for the basic comparison-functions, that is, the generic ones and the
 * concrete defined ones for numbers, strings and boolean.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Conditions.hpp"
#include "Comparisons.hpp"

#include "Elob.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Conditions::define(proc);
  Comparisons::define(proc);

  Elob val;
  #define DO_CHECK(instr, res) \
    val=proc.evalClassic(instr, proc.getBooleanType()); \
    check(val.getBoolean()==res, \
      "Comparison %s value mismatch!", instr);

  // Classic for integers
  DO_CHECK("(Gleichheit 5 5)", true)
  DO_CHECK("(Gleichheit 5 7)", false)
  DO_CHECK("(KleinerAls 5 7)", true)
  DO_CHECK("(KleinerAls 5 5)", false)

  // Classic for numbers
  DO_CHECK("(Gleichheit 5. 5.)", true)
  DO_CHECK("(Gleichheit 5 5.)", true)
  DO_CHECK("(Gleichheit 5. 5.0001)", false)
  DO_CHECK("(KleinerAls 5 5.000001)", true)
  DO_CHECK("(KleinerAls 5 4.99999)", false)

  // Classic for strings
  DO_CHECK("(Gleichheit \"abc\" \"abc\")", true)
  DO_CHECK("(Gleichheit \"abc\" \"abcde\")", false)
  DO_CHECK("(Gleichheit \"abcde\" \"abc\")", false)
  DO_CHECK("(KleinerAls \"abc\" \"abc\")", false)
  DO_CHECK("(KleinerAls \"a\" \"aa\")", true)
  DO_CHECK("(KleinerAls \"abce\" \"abcd\")", false)

  // Classic for boolean
  DO_CHECK("(Gleichheit wahr wahr)", true)
  DO_CHECK("(Gleichheit wahr falsch)", false)
  DO_CHECK("(Gleichheit falsch falsch)", true)

  // Generic comparisons
  DO_CHECK("(Ungleichheit 4. 4)", false)
  DO_CHECK("(Ungleichheit \"abc\" \"def\")", true)
  DO_CHECK("(GroeszerAls \"Patrica\" \"Daniel\")", true)
  DO_CHECK("(GroeszerAls 5 5)", false)
  DO_CHECK("(KleinerGleich 5 5)", true)
  DO_CHECK("(KleinerGleich 4.0 5.1)", true)
  DO_CHECK("(KleinerGleich 5.1 4)", false)
  DO_CHECK("(GroeszerGleich 5 5)", true)
  DO_CHECK("(GroeszerGleich 4.0 5.1)", false)
  DO_CHECK("(GroeszerGleich 5.1 4)", true)

  return EXIT_SUCCESS;
}
