/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test for the boolean type, logic operations and the if-conditional-function
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Conditions.hpp"

#include "Elob.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Conditions::define(proc);

  Elob val;

  // Check boolean type
  val=Elob::forBoolean(true, proc);
  check(val.getBoolean(), "Boolean-Elob should be true!");
  val=Elob::forBoolean(false, proc);
  check(!val.getBoolean(), "Boolean-Elob should be false!");

  // Check logic operators
  #define LOGIC_CHECK(instr, res) \
    val=proc.evalClassic(instr, proc.getBooleanType()); \
    check(val.getBoolean()==res, \
      "Logic-operation %s value mismatch!", instr);
  LOGIC_CHECK("(Verneinung wahr)", false)
  LOGIC_CHECK("(Verneinung falsch)", true)
  LOGIC_CHECK("(Konjunktion falsch falsch)", false)
  LOGIC_CHECK("(Konjunktion falsch wahr)", false)
  LOGIC_CHECK("(Konjunktion wahr wahr)", true)
  LOGIC_CHECK("(Disjunktion falsch falsch)", false)
  LOGIC_CHECK("(Disjunktion wahr falsch)", true)
  LOGIC_CHECK("(Disjunktion wahr wahr)", true)

  // Check if-function and constants the "classic" way
  #define CLASSIC_CHECK(instr, restype, res) \
    val=proc.evalClassic(instr, proc.get##restype##Type()); \
    check(val.get##restype()==res, "Result value mismatch");
  CLASSIC_CHECK("(_if wahr 5 \"abc\")", Integer, 5)
  CLASSIC_CHECK("(_if falsch 5 \"abc\")", String, "abc")

  return EXIT_SUCCESS;
}
