/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests the German language frontend for action-literals / action definitions.
 */

#include "DeComparator.hpp"

#include <cstdlib>

using namespace lispel;
using namespace tests;

using namespace std;

int main()
{
  DeComparator proc;

  /* Action literals */
  //proc.check("spiele!?", "(! spiele)");
  proc.check("gib_aus mit \"hello\"!?", "(! gib_aus \"hello\")");
  proc.check("singe mit der Summe von a und 3, dem Sinus c und 42!?",
    "(! singe (Summe a 3) (Sinus c) 42)");
  proc.check("xxx mit 3 und setze mit c!!?",
    "(! xxx 3 (! setze c))");

  /* Action execution */
  proc.check("spiele mit y und x!!", "(spiele y x)");
  proc.check("mache fuer x y, sonst z!!", "(_do (_if x y z))");

  /* Action definition */
  proc.check("spiele bedeutet mit der Zahl x: spiele mit x!.",
    "(_action spiele ((Zahl x)) ((spiele x)))");
  proc.check("spiele bedeutet mit der Zahl a: "
      " singe mit a und 3! spiele mit 5! mache a!.",
    "(_action spiele ((Zahl a)) ((singe a 3) (spiele 5) (_do a)))");
  proc.check("spiele bedeutet mit der Zahl a und der Zahl b: mache nichts!.",
    "(_action spiele ((Zahl a) (Zahl b)) ((_do nichts)))");
  proc.check("spiele bedeutet mit der Zahl a, der Zahl b und der Zahl x: "
      " xxx mit a, b und x!.",
    "(_action spiele ((Zahl a) (Zahl b) (Zahl x)) ((xxx a b x)))");

  /* Action bodies with variable definitions */
  proc.check("spiele bedeutet mit der Zahl x: sei Pi 3.14159!.",
    "(_action spiele ((Zahl x)) ((_constant Pi 3.14159)))");
  proc.check("spiele bedeutet mit der Zahl x: "
    " singe mit y! sei Pi 3.14159! spiele mit Pi!.",
    "(_action spiele ((Zahl x)) "
    " ((singe y) (_constant Pi 3.14159) (spiele Pi)))");

  return EXIT_SUCCESS;
}
