/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests the shell for depel (the main command processor, that is, not only
 * interactively used).
 */

#include "Test.hpp"

#include "DepelShell.hpp"

#include "Cell.hpp"

#include <cstdlib>
#include <sstream>
#include <string>
#include <cmath>

using namespace lispel;
using namespace ellisp;
using namespace depel;
using namespace tests;

static const char* program(
  "Zahl n gleich Zahl m ist die Gleichheit von n und m. "
  "Zahl n mal Zahl m ist das Produkt von n und m. "

  "Der Mittelwert von der Zahl a, der Zahl b und der Zahl c ist "
  " der Quotient von der Summe von a, b und c und 3 . "
  "Das Quadrat von der Zahl n ist n mal n. "
  "Die Varianz von der Zahl a, der Zahl b und der Zahl c ist "
  " der Quotient von der Summe von "
  "  dem Quadrat der Differenz von a und dem Mittelwert von a, b und c, "
  "  dem Quadrat der Differenz von b und dem Mittelwert von a, b und c und "
  "  dem Quadrat der Differenz von c und dem Mittelwert von a, b und c "
  " und 3 . "
  "Die Fakultaet von der Ganzzahl n ist "
  " fuer n gleich 0 1, "
  " sonst n mal der Fakultaet der Differenz von n und 1 . "

  "das Quadrat 5? "
  "die Varianz von 2, 11 und 5? "
  "die Fakultaet 5? "
  );

double variance(double a, double b, double c)
{
  const double mean((a+b+c)/3.);
  #define SQ(x) ((x)*(x))
  return (SQ(a-mean)+SQ(b-mean)+SQ(c-mean))/3.;
}

int main()
{
  std::string programStr(program);
  std::istringstream in(programStr);
  std::ostringstream out;
  DepelShell shell(in, out);

  // Run the program
  shell.run();

  // Finally, check the result
  std::istringstream outIn(out.str());
  Cell::intType quad, fak;
  Cell::numberType var;
  outIn >> quad;
  check(quad==5*5, "Quadrat 5 calculated wrong, is %d", quad);
  outIn >> var;
  check(std::abs(var-variance(2, 5, 11))<1e-6,
    "Varianz is wrong: %f, should be %f", var, variance(2, 5, 11));
  outIn >> fak;
  check(fak==1*2*3*4*5, "Fakultaet 5 wrong, is %d (should be 120)", fak);

  return EXIT_SUCCESS;
}
