/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests the German language frontend with respect to values; that is, literal
 * values, function calls, casts and definition of constants.
 */

#include "Test.hpp"
#include "DeComparator.hpp"

#include "Parser.hpp"

#include <cstdlib>

using namespace lispel;
using namespace tests;
using namespace depel;

using namespace std;

int main()
{
  DeComparator proc;

  proc.check("a?", "a");
  proc.check("-5?", "-5");
  proc.check("\"abc\"?", "\"abc\"");
  proc.check("[(Summe 5 3)]?", "(Summe 5 3)");
  proc.check("der Sinus 5?", "(Sinus 5)");
  proc.check("die Potenz von a und 3?", "(Potenz a 3)");
  proc.check("der Sinus die Summe von a und b?", "(Sinus (Summe a b))");
  proc.check("die Summe von 5, dem Produkt von a und b, 3 und 2?",
    "(Summe 5 (Produkt a b) 3 2)");
  proc.check("die Summe von 5, dem Sinus a und 3?", "(Summe 5 (Sinus a) 3)");
  proc.check("die Summe von 5 und 3.5 als Ganzzahl?",
    "(Summe 5 (_cast Ganzzahl 3.5))");
  proc.check("die Summe als Ganzzahl von 5 und 3.5?",
    "(_cast Ganzzahl (Summe 5 3.5))");
  proc.check("der Sinus als Ganzzahl 1?",
    "(_cast Ganzzahl (Sinus 1))");
  proc.check("Sei Pi 3.14159.", "(_constant Pi 3.14159)");

  // Check EOF throwing
  bool eof(false);
  try
  {
    proc.check("", "0"); // Dummy call just to call the parser
  } catch(const Parser::EndOfInput& exc)
  {
    eof=true;
  }
  check(eof, "EOF not thrown by parser");

  return EXIT_SUCCESS;
}
