/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* funEval tests the basic calling and evaluation of functions, built-in
 * ones.  Thereby it also tests transformation of Cell literals (number,
 * integer, string) to Elobs.
 */

#include "Test.hpp"

#include "CellTypes.hpp"

#include "Cell.hpp"
#include "List.hpp"
#include "Runtime.hpp"
#include "Elob.hpp"
#include "Function.hpp"
#include "Atom.hpp"

#include <cstdlib>
#include <sstream>
#include <memory>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

// The function call to evaluate as strings
const char* IN("(add (inc 4) .25)");
const Cell::numberType OUT((4+1)+.25);

// The callback for inc
Elob calcInc(const Runtime& rt, const Arguments& args)
{
  check(args.size()==1, "Inc called with %d arguments!", args.size());
  check(args.front().getType()==rt.getIntegerType(), "Inc not called with int");
  return Elob::forInteger(args.front().getInteger()+1, rt);
}

// The callback for add
Elob calcAdd(const Runtime& rt, const Arguments& args)
{
  check(args.size()==2, "Add called with %d arguments!", args.size());
  check(args.front().getType()==rt.getNumberType(), "Add not called with num!");
  check(args.back().getType()==rt.getNumberType(), "Add not called with num!");
  return Elob::forNumber(args.front().getNumber()+args.back().getNumber(), rt);
}

int main()
{
  Runtime proc;
  CellTypes::define(proc);

  Signature sig;
  sig.push_back(proc.getIntegerType());
  proc.addFunction(Atom("inc"),
    new CallbackFunction(proc, sig, &calcInc));

  sig.clear();
  sig.push_back(proc.getNumberType());
  sig.push_back(proc.getNumberType());
  proc.addFunction(Atom("add"),
    new CallbackFunction(proc, sig, &calcAdd));

  istringstream in(IN);
  auto_ptr<List> lst(List::input(in));
  Elob val(proc.evaluate(lst.get()));

  check(val.getType()==proc.getNumberType(), "Final value is of wrong type!");
  check(val.getNumber()==OUT, "Final value is wrong: %f instead of %f!",
    val.getNumber(), OUT);
  
  return EXIT_SUCCESS;
}
