/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008,2012  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Special test that the (if xxx a b) expression does lazy-evaluation of a and
 * b, thus allowing for recursion like
 * (_function fact ((Ganzzahl n))
 *  (if (Gleichheit n 0)
 *    1
 *    (Produkt n (fact (Differenz n 1)))))
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Conditions.hpp"
#include "Comparisons.hpp"
#include "Arithmetic.hpp"

#include "Elob.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

template<int n>
  class factorial
{
  public:
    static const int val;
};
template<int n>
  const int factorial<n>::val=n*factorial<n-1>::val;
template<>
  class factorial<0>
{
  public:
    static const int val=1;
};

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);
  Conditions::define(proc);
  Comparisons::define(proc);

  Elob val;

  // Define recursive factorial function
  proc.procClassic("(_function factorial ((Ganzzahl n))"
    " (_if (Gleichheit n 0)"
    "  1"
    "  (Produkt n (factorial (Differenz n 1)))))");

  // Test it
  #define CHECK(n) \
    val=proc.evalClassic("(factorial " #n ")", proc.getIntegerType()); \
    check(val.getInteger()==factorial<n>::val, "Factorial of " #n " is wrong!");
  CHECK(0)
  CHECK(1)
  CHECK(5)
  CHECK(20)

  return EXIT_SUCCESS;
}
