/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests lazy-evaluation of side-effect-sequence-creation; this is needed if,
 * for instance, a side-effect prior to some other sets a variable used later
 * as argument.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Variables.hpp"
#include "SideEffects.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Variables::define(proc);
  SideEffects::define(proc);

  // Define two variables
  proc.procClassic("(_constant a (Variable 0))");
  proc.procClassic("(_constant b (Variable 0))");

  // Now, increment a and set b=a, this should also set b to 1! 
  proc.procClassic("(_do (_sequence (! setze a 1) (! setze b (Wert a))))");

  // Check the value of b
  Elob val(proc.evalClassic("(Wert b)", proc.getIntegerType()));
  check(val.getInteger()==1, "Sequence not evaluated lazyly!");

  return EXIT_SUCCESS;
}
