/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests the side-effect as value system; that is, literal actions in
 * Lispel-code and the library functions for sequence-creation and the
 * do-construct.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Variables.hpp"
#include "Streams.hpp"
#include "SideEffects.hpp"
#include "Arithmetic.hpp"
#include "Conditions.hpp"

#include <cstdlib>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);
  Conditions::define(proc);
  Variables::define(proc);
  Streams::define(proc);
  SideEffects::define(proc);

  Elob val;

  // Define a variable and literal action to increment it
  proc.procClassic("(_constant x (Variable 0))");
  proc.procClassic("(_action inc ((Variable a))"
    " ((setze a (Summe (Wert a) 1))))");
  proc.procClassic("(_constant doInc (! inc x))");

  #define CHECK_VALUE(shouldBe, action) \
    val=proc.evalClassic("(Wert x)", proc.getIntegerType()); \
    check(val.getInteger()==shouldBe, action " done wrong!");

  // Execute it thrice (in sequence)
  proc.procClassic("(_do (Sequenz doInc doInc doInc))");
  CHECK_VALUE(3, "Increment-sequence")

  // Execute conditionally (that's what do is meant for)
  proc.procClassic("(_do (_if wahr doInc nichts))");
  CHECK_VALUE(4, "If (true)")
  proc.procClassic("(_do (_if falsch doInc nichts))");
  CHECK_VALUE(4, "If (false)")

  // Just for fun, print out the literal action
  proc.procClassic("(gib-aus stdout doInc)");
  proc.procClassic("(schreibe stdout \"\n\")");
  proc.procClassic("(gib-aus stdout nichts)");
  proc.procClassic("(schreibe stdout \"\n\")");

  return EXIT_SUCCESS;
}
