/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests definition and working of userdefined, "dynamic" actions.  Defines
 * one per List and then checks that it works as expected.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Arithmetic.hpp"
#include "Variables.hpp"

#include <cstdlib>
#include <cmath>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);
  Variables::define(proc);

  // Define a user-action
  proc.procClassic("(_action SquareInc ((Variable x))"
    " ((setze x (Produkt (Wert x) (Wert x))) (setze x (Summe (Wert x) 1))))");

  // Test it
  proc.procClassic("(_constant var (Variable 42))");
  proc.procClassic("(SquareInc var)");
  Elob val(proc.evalClassic("(Wert var)", proc.getIntegerType()));
  check(val.getInteger()==42*42+1, "var processed wrong: %d, should be %d!",
    val.getInteger(), 42*42+1);

  // User-action with in-scope variable declaration
  proc.procClassic("(_action XXX () ((_constant Pi 3.14159)))");
  proc.procClassic("(XXX)");

  return EXIT_SUCCESS;
}
