/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Test-case for definition and usage of user-classes; this includes:
 * - new type defined
 * - defined accessor methods
 * - defined constructor methods
 * - up-casting to super-types
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Strings.hpp"
#include "Arithmetic.hpp"

#include <cstdlib>
#include <iostream>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Strings::define(proc);
  Arithmetic::define(proc);

  Elob val;

  // Define user-classes forming a basic type-hierarchy
  proc.procClassic("(_class Person ((String Name)) ((Ganzzahl Alter)))");
  proc.procClassic("(_class Mann ((Zahl Verdienst) (Person)) ())");
  proc.procClassic("(_class Frau ((Person)) ((String Wohnzimmerfarbe)))");

  // Construct one object of Mann/Frau
  proc.procClassic("(_constant Daniel "
    " (Neu-Mann 123.5 (Neu-Person \"Daniel\" 19)))");
  proc.procClassic("(_constant Trish "
    " (Neu-Frau (Neu-Person \"Patrica\" 20) \"Gelb gepunktet\"))");

  // Output them for fun
  /*
  val=proc.evalClassic("Daniel", proc.queryType(Atom("Mann")));
  cout << val;
  val=proc.evalClassic("Trish", proc.queryType(Atom("Frau")));
  cout << val;
  */

  // Test accessor methods
  val=proc.evalClassic("(Mann-Verdienst Daniel)", proc.getNumberType());
  check(val.getNumber()==123.5, "Accessor of Mann::Verdienst was wrong!");
  val=proc.evalClassic("(Frau-Wohnzimmerfarbe Trish)", proc.getStringType());
  check(val.getString()=="Gelb gepunktet", 
    "Accessor of Frau::Wohnzimmerfarbe was wrong!");

  // And some polymorphism/upcasting tests
  val=proc.evalClassic("(Person-Alter Daniel)", proc.getIntegerType());
  check(val.getInteger()==19,
    "Upcast Mann => Person went wrong, got %d instead of 19", val.getInteger());
  val=proc.evalClassic("(Verkettung Daniel Trish)", proc.getStringType());
  check(val.getString()=="DanielPatrica", "Upcasting for Verkettung failed!");

  return EXIT_SUCCESS;
}
