/*

    Lispel, typed German language Lisp.
    Copyright (C) 2008  Daniel Kraft <d@domob.eu>
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

/* Tests definition and working of userdefined, "dynamic" functions.  Defines
 * one per List and then checks that it works as expected.
 */

#include "Test.hpp"
#include "ClassicEval.hpp"

#include "Arithmetic.hpp"

#include <cstdlib>
#include <cmath>

using namespace lispel;
using namespace ellisp;
using namespace ellib;
using namespace tests;

using namespace std;

int main()
{
  ClassicEval proc;
  Arithmetic::define(proc);

  // Define some functions
  proc.procClassic("(_function Pi () 3.14159)");
  proc.procClassic("(_function Square ((Ganzzahl n)) (Produkt n n))");
  proc.procClassic("(_function XXX ((Ganzzahl a) (Zahl n))\n"
    " (Summe (Square a) n))");

  /* Test those */

  Elob val;

  val=proc.evalClassic("(Pi)", proc.getNumberType());
  check(abs(val.getNumber()-3.14159)<1e-10, "Pi-function wrong");

  val=proc.evalClassic("(Square 5)", proc.getIntegerType());
  check(val.getInteger()==5*5, "Square-function wrong");

  val=proc.evalClassic("(XXX 9 3.5)", proc.getNumberType());
  check(abs(val.getNumber()-(9*9+3.5))<1e-10, "XXX-function wrong");

  return EXIT_SUCCESS;
}
